@@ -5,13 +5,19 @@ use POSIX;
 chop($system_arch = `uname -m`);
 $package_dir = "/var/db/pkg";
 
+sub list_package_system_commands
+{
+return ("pkg_info", "pkg_add");
+}
+
 # list_packages([package]*)
 # Fills the array %packages with a list of all packages
 sub list_packages
 {
 local $i = 0;
-local $arg = @_ ? join(" ", @_) : "-a";
-open(PKGINFO, "pkg_info -I $arg |");
+local $arg = @_ ? join(" ", map { quotemeta($_) } @_) : "-a";
+%packages = ( );
+&open_execute_command(PKGINFO, "pkg_info -I $arg", 1, 1);
 while(<PKGINFO>) {
 	if (/^(\S+)\s+(.*)/) {
 		$packages{$i,'name'} = $1;
@@ -29,7 +35,8 @@ return $i;
 #  name, class, description, arch, version, vendor, installtime
 sub package_info
 {
-local $out = `pkg_info $_[0] 2>&1`;
+local $qm = quotemeta($_[0]);
+local $out = &backquote_command("pkg_info $qm 2>&1", 1);
 return () if ($?);
 local @rv = ( $_[0] );
 push(@rv, "");
@@ -37,11 +44,19 @@ push(@rv, $out =~ /Description:\n([\0-\177]*\S)/i ? $1 : $text{'bsd_unknown'});
 push(@rv, $system_arch);
 push(@rv, $_[0] =~ /-([^\-]+)$/ ? $1 : $text{'bsd_unknown'});
 push(@rv, "OpenBSD");
-local @st = stat("$package_dir/$_[0]");
+local @st = stat(&translate_filename("$package_dir/$_[0]"));
 push(@rv, @st ? ctime($st[9]) : $text{'bsd_unknown'});
 return @rv;
 }
 
+# virtual_package_info(package)
+# Returns an array of package information for a virtual package, usually called
+# if "package_info" returns nothing.
+sub virtual_package_info
+{
+return ( );
+}
+
 # check_files(package)
 # Fills in the %files array with information about the files belonging
 # to some package. Values in %files are  path type user group mode size error
@@ -49,19 +64,21 @@ sub check_files
 {
 local $i = 0;
 local $file;
-open(PKGINFO, "pkg_info -L $_[0] |");
+local $qm = quotemeta($_[0]);
+&open_execute_command(PKGINFO, "pkg_info -L $qm", 1, 1);
 while($file = <PKGINFO>) {
 	$file =~ s/\r|\n//g;
 	next if ($file !~ /^\//);
-	local @st = stat($file);
+	local $real = &translate_filename($file);
+	local @st = stat($real);
 	$files{$i,'path'} = $file;
-	$files{$i,'type'} = -l $file ? 3 :
-			    -d $file ? 1 : 0;
+	$files{$i,'type'} = -l $real ? 3 :
+			    -d $real ? 1 : 0;
 	$files{$i,'user'} = getpwuid($st[4]);
 	$files{$i,'group'} = getgrgid($st[5]);
 	$files{$i,'mode'} = sprintf "%o", $st[2] & 07777;
 	$files{$i,'size'} = $st[7];
-	$files{$i,'link'} = readlink($file);
+	$files{$i,'link'} = readlink($real);
 	$i++;
 	}
 return $i;
@@ -76,7 +93,8 @@ sub installed_file
 local (%packages, $file, $i, @pkgin);
 local $n = &list_packages();
 for($i=0; $i<$n; $i++) {
-	open(PKGINFO, "pkg_info -L $packages{$i,'name'} |");
+	local $qm = quotemeta($packages{$i,'name'});
+	&open_execute_command(PKGINFO, "pkg_info -L $qm", 1,1);
 	while($file = <PKGINFO>) {
 		$file =~ s/\r|\n//g;
 		if ($file eq $_[0]) {
@@ -87,15 +105,16 @@ for($i=0; $i<$n; $i++) {
 	close(PKGINFO);
 	}
 if (@pkgin) {
-	local @st = stat($_[0]);
+	local $real = &translate_filename($_[0]);
+	local @st = stat($real);
 	$file{'path'} = $_[0];
-	$file{'type'} = -l $_[0] ? 3 :
-			-d $_[0] ? 1 : 0;
+	$file{'type'} = -l $real ? 3 :
+			-d $real ? 1 : 0;
 	$file{'user'} = getpwuid($st[4]);
 	$file{'group'} = getgrgid($st[5]);
 	$file{'mode'} = sprintf "%o", $st[2] & 07777;
 	$file{'size'} = $st[7];
-	$file{'link'} = readlink($_[0]);
+	$file{'link'} = readlink($real);
 	$file{'packages'} = join(" ", @pkgin);
 	return 1;
 	}
@@ -108,7 +127,8 @@ else {
 sub is_package
 {
 local ($desc, $contents);
-open(TAR, "gunzip -c $_[0] | tar tf - |");
+local $qm = quotemeta($_[0]);
+&open_execute_command(TAR, "gunzip -c $qm | tar tf -", 1, 1);
 while(<TAR>) {
 	$desc++ if (/^\+DESC/);
 	$contents++ if (/^\+CONTENTS/);
@@ -122,20 +142,20 @@ return $desc && $contents;
 #  package description
 sub file_packages
 {
-local $temp = &tempname();
-mkdir($temp, 0700);
-system("cd $temp ; gunzip -c $_[0] | tar xf - +CONTENTS +COMMENT >/dev/null 2>&1");
+local $temp = &transname();
+&make_dir($temp, 0700);
+local $qm = quotemeta($_[0]);
+&execute_command("cd $temp && gunzip -c $qm | tar xf - +CONTENTS +COMMENT");
 local ($comment, $name);
-open(COMMENT, "$temp/+COMMENT");
+&open_readfile(COMMENT, "$temp/+COMMENT");
 ($comment = <COMMENT>) =~ s/\r|\n//g;
 close(COMMENT);
-open(CONTENTS, "$temp/+CONTENTS");
+&open_readfile(CONTENTS, "$temp/+CONTENTS");
 while(<CONTENTS>) {
 	$name = $1 if (/^\@name\s+(\S+)/);
 	}
 close(CONTENTS);
-unlink("$temp/+COMMENT", "$temp/+CONTENTS");
-rmdir($temp);
+&unlink_file($temp);
 return ( "$name $comment" );
 }
 
@@ -143,22 +163,22 @@ return ( "$name $comment" );
 # Outputs HTML for choosing install options
 sub install_options
 {
-print "<tr> <td><b>$text{'bsd_scripts'}</b></td>\n";
-print "<td><input type=radio name=scripts value=0 checked> $text{'yes'}\n";
-print "<input type=radio name=scripts value=1> $text{'no'}</td>\n";
+print &ui_table_row($text{'bsd_scripts'},
+	&ui_radio("scripts", 0, [ [ 0, $text{'yes'} ], [ 1, $text{'no'} ] ]));
 
-print "<td><b>$text{'bsd_force'}</b></td>\n";
-print "<td><input type=radio name=force value=1> $text{'yes'}\n";
-print "<input type=radio name=force value=0 checked> $text{'no'}</td> </tr>\n";
+print &ui_table_row($text{'bsd_force'},
+	&ui_yesno_radio("force", 1));
 }
 
 # install_package(file, package)
 # Installs the package in the given file, with options from %in
 sub install_package
 {
-local $args = ($in{"scripts"} ? " -I" : "").
-	      ($in{"force"} ? " -f" : "");
-local $out = &backquote_logged("pkg_add $args $_[0] 2>&1");
+local $in = $_[2] ? $_[2] : \%in;
+local $args = ($in->{"scripts"} ? " -I" : "").
+	      ($in->{"force"} ? " -f" : "");
+local $qm = quotemeta($_[0]);
+local $out = &backquote_logged("pkg_add $args $qm 2>&1");
 if ($?) {
 	return "<pre>$out</pre>";
 	}
@@ -169,16 +189,20 @@ return undef;
 # Totally remove some package
 sub delete_package
 {
-local $out = &backquote_logged("pkg_delete $_[0] 2>&1");
+local $qm = quotemeta($_[0]);
+local $out = &backquote_logged("pkg_delete $qm 2>&1");
 if ($?) { return "<pre>$out</pre>"; }
 return undef;
 }
 
 sub package_system
 {
-return "OpenBSD Package Manager";
+return &text('bsd_manager', "OpenBSD");
 }
 
-
+sub package_help
+{
+return "pkg_add pkg_info pkg_delete";
+}
 
 1;
